@@ -34,6 +34,7 @@ import platform
 import pygtk
 import subprocess
 import sys
+import traceback
 import uuid
 import xdg.Exceptions
 
@@ -81,13 +82,13 @@ try:
     from ctypes import cdll
     libutempter = cdll.LoadLibrary('libutempter.so.0')
     if libutempter is not None:
-        # We absolutly need to remove the old tty from the utmp !!!!!!!!!!
+        # We absolutely need to remove the old tty from the utmp !!!
         at_exit_call(libutempter.utempter_remove_added_record)
 except Exception as e:
     libutempter = None
-    sys.stderr.write('[WARN] Unable to load the library libutempter !\n')
-    sys.stderr.write('[WARN] The <wall> command will not work in guake !\n')
-    sys.stderr.write('[WARN] ' + str(e) + '\n')
+    sys.stderr.write("[WARN] Unable to load the library libutempter !\n")
+    sys.stderr.write("[WARN] The <wall> command will not work in guake !\n")
+    sys.stderr.write("[WARN] " + str(e) + '\n')
 
 instance = None
 RESPONSE_FORWARD = 0
@@ -129,10 +130,10 @@ class PromptQuitDialog(gtk.MessageDialog):
             gtk.MESSAGE_QUESTION, gtk.BUTTONS_YES_NO)
 
         if tabs == -1:
-            primary_msg = _('Do you want to close the tab?')
+            primary_msg = _("Do you want to close the tab?")
             tab_str = ''
         else:
-            primary_msg = _('Do you really want to quit Guake?')
+            primary_msg = _("Do you really want to quit Guake?")
             if tabs == 1:
                 tab_str = _(" and one tab open")
             else:
@@ -174,13 +175,14 @@ class Guake(SimpleGladeApp):
         self.custom_command_menuitem = None
 
         # trayicon! Using SVG handles better different OS trays
-        img = pixmapfile('guake-tray.svg')
-
+        # img = pixmapfile('guake-tray.svg')
+        # trayicon!
+        img = pixmapfile('guake-tray.png')
         try:
             import appindicator
         except ImportError:
             self.tray_icon = gtk.status_icon_new_from_file(img)
-            self.tray_icon.set_tooltip(_('Guake Terminal'))
+            self.tray_icon.set_tooltip(_("Guake Terminal"))
             self.tray_icon.connect('popup-menu', self.show_menu)
             self.tray_icon.connect('activate', self.show_hide)
         else:
@@ -297,23 +299,6 @@ class Guake(SimpleGladeApp):
         self.abbreviate = False
         self.was_deleted_tab = False
 
-        def tabs_scrollbar_hide(hscrollbar):
-            self.get_widget('event-tabs').set_property('height_request', 10)
-            if self.abbreviate and self.was_deleted_tab:
-                self.was_deleted_tab = False
-                self.abbreviate = False
-                self.recompute_tabs_titles()
-
-        def tabs_scrollbar_show(hscrollbar):
-            self.get_widget('event-tabs').set_property('height_request', -1)
-            if self.client.get_bool(KEY("/general/abbreviate_tab_names")):
-                self.abbreviate = True
-                self.recompute_tabs_titles()
-
-        tabs_scrollbar = self.get_widget('tabs-scrolledwindow').get_hscrollbar()
-        tabs_scrollbar.connect('hide', tabs_scrollbar_hide)
-        tabs_scrollbar.connect('show', tabs_scrollbar_show)
-
         # Flag to prevent guake hide when window_losefocus is true and
         # user tries to use the context menu.
         self.showing_context_menu = False
@@ -377,9 +362,9 @@ class Guake(SimpleGladeApp):
             # Pop-up that shows that guake is working properly (if not
             # unset in the preferences windows)
             guake.notifier.show_message(
-                _('Guake Terminal'),
-                _('Guake is now running,\n'
-                  'press <b>%s</b> to use it.') % xml_escape(label), filename)
+                _("Guake Terminal"),
+                _("Guake is now running,\n"
+                  "press <b>{!s}</b> to use it.").format(xml_escape(label)), filename)
 
     # load the custom commands infrastucture
     def load_custom_commands(self):
@@ -528,6 +513,16 @@ class Guake(SimpleGladeApp):
                 else:
                     t.set_background_transparent(True)
 
+    def set_bg_image(self, image, tab=None):
+        """Set the background image of `tab' or the current tab to `bgcolor'."""
+        if not self.notebook.has_term():
+            self.add_tab()
+        index = tab or self.notebook.get_current_page()
+        for terminal in self.notebook.get_terminals_for_tab(index):
+            if image and os.path.exists(image):
+                terminal.set_background_image_file(image)
+                terminal.set_background_transparent(False)
+
     def set_bgcolor(self, bgcolor, tab=None):
         """Set the background color of `tab' or the current tab to `bgcolor'."""
         if not self.notebook.has_term():
@@ -580,8 +575,8 @@ class Guake(SimpleGladeApp):
                 current_vte.feed_child(command)
 
     def on_resizer_drag(self, widget, event):
-        """Method that handles the resize drag. It does not actuall
-        moves the main window. It just set the new window size in
+        """Method that handles the resize drag. It does not actually
+        move the main window. It just sets the new window size in
         gconf.
         """
         (x, y), mod = event.device.get_state(widget.window)
@@ -630,10 +625,11 @@ class Guake(SimpleGladeApp):
 
         value = self.client.get_bool(KEY('/general/window_losefocus'))
         visible = window.get_property('visible')
-        if value and visible:
+        if visible:
             self.losefocus_time = gtk.gdk.x11_get_server_time(
                 self.window.window)
-            self.hide()
+            if value:
+                self.hide()
 
     def show_menu(self, status_icon, button, activate_time):
         """Show the tray icon menu.
@@ -717,6 +713,16 @@ class Guake(SimpleGladeApp):
             menu.popup(None, None, None, 3, event.get_time())
         self.set_terminal_focus()
 
+    def middle_button_click(self, target, event):
+        """Closes a tab with a middle click
+        """
+        if event.button == 2 and event.type == gtk.gdk.BUTTON_PRESS:
+            previously_selected_tab = self.get_selected_tab()
+            target.activate_tab()
+            target_position = self.get_selected_tab()
+            self.select_tab(previously_selected_tab)
+            self.delete_tab(target_position)
+
     def show_about(self, *args):
         """Hides the main window and creates an instance of the About
         Dialog.
@@ -753,11 +759,21 @@ class Guake(SimpleGladeApp):
 
         event_time = self.hotkeys.get_current_event_time()
 
-        if self.losefocus_time and \
-                self.losefocus_time >= event_time and \
-                (self.losefocus_time - event_time) < 10:
-            self.losefocus_time = 0
-            return
+        if self.client.get_bool(KEY('/general/focus_if_open')) and \
+                not self.client.get_bool(KEY('/general/window_losefocus')):
+            if self.losefocus_time and self.losefocus_time < event_time:
+                if self.window.window and self.window.get_property('visible'):
+                    log.debug("DBG: Restoring the focus to the terminal")
+                    self.window.window.focus()
+                    self.set_terminal_focus()
+                    self.losefocus_time = 0
+                    return
+        elif self.losefocus_time and \
+                self.client.get_bool(KEY('/general/window_losefocus')):
+            if self.losefocus_time >= event_time and \
+                    (self.losefocus_time - event_time) < 10:
+                self.losefocus_time = 0
+                return
 
         # limit rate at which the visibility can be toggled.
         if self.prev_showhide_time and event_time and \
@@ -834,9 +850,9 @@ class Guake(SimpleGladeApp):
         if self.selected_color is None:
             self.selected_color = getattr(self.window.get_style(), "light")[int(gtk.STATE_SELECTED)]
 
-            # Reapply the tab color to all button in the tab list, since at least one don't have the
-            # select color set. This needs to happen AFTER the first show_all, since before the gtk
-            # has not loaded the right colors yet.
+            # Reapply the tab color to all button in the tab list, since at least one doesn't have
+            # the select color set. This needs to happen AFTER the first show_all, since before,
+            # gtk has not loaded the right colors yet.
             for tab in self.tabs.get_children():
                 if isinstance(tab, gtk.RadioButton):
                     tab.modify_bg(gtk.STATE_ACTIVE, gtk.gdk.Color(str(self.selected_color)))
@@ -845,7 +861,7 @@ class Guake(SimpleGladeApp):
         self.printDebug("Moving window to: %r", window_rect)
         self.window.move(window_rect.x, window_rect.y)
 
-        # this work arround an issue in fluxbox
+        # this works around an issue in fluxbox
         if not self.is_fullscreen:
             self.client.notify(KEY('/general/window_height'))
 
@@ -895,6 +911,7 @@ class Guake(SimpleGladeApp):
         self.client.notify(KEY('/style/background/color'))
 
         self.printDebug("Current window position: %r", self.window.get_position())
+        self.execute_hook('show')
 
     def hide_from_remote(self):
         """
@@ -1002,10 +1019,12 @@ class Guake(SimpleGladeApp):
         if self.is_using_unity():
 
             # For Ubuntu 12.10 and above, try to use dconf:
-            # see if unity dock is hiden => unity_hide
-            # and the width of unity dock. => unity_dock
+            # see if unity dock is hidden => unity_hide
+            # and the width of unity dock => unity_dock
+            # and the position of the unity dock. => unity_pos
             found = False
             unity_hide = 0
+            unity_pos = "Left"
             # float() conversion might mess things up. Add 0.01 so the comparison will always be
             # valid, even in case of float("10.10") = 10.099999999999999
             if float(platform.linux_distribution()[1]) + 0.01 >= 12.10:
@@ -1016,6 +1035,9 @@ class Guake(SimpleGladeApp):
                     unity_dock = int(subprocess.check_output(
                         ['/usr/bin/dconf', 'read',
                          '/org/compiz/profiles/unity/plugins/unityshell/icon-size']) or "48")
+                    unity_pos = subprocess.check_output(
+                        ['/usr/bin/dconf', 'read',
+                         '/com/canonical/unity/launcher/launcher-position']) or "Left"
                     found = True
                 except:
                     # in case of error, just ignore it, 'found' will not be set to True and so
@@ -1030,9 +1052,11 @@ class Guake(SimpleGladeApp):
                 unity_dock = unity_icon_size + 17
 
             # launcher_hide_mode = 1 => autohide
-            if unity_hide != 1:
-                self.printDebug("correcting window width because of launcher width %s "
-                                "(from %s to %s)",
+            # only adjust guake window width if Unity dock is positioned "Left" or "Right"
+            if unity_hide != 1 and (unity_pos == "Left" or unity_pos == "Right"):
+                self.printDebug("correcting window width because of launcher position %s "
+                                "and width %s (from %s to %s)",
+                                unity_pos,
                                 unity_dock,
                                 window_rect.width,
                                 window_rect.width - unity_dock)
@@ -1066,7 +1090,7 @@ class Guake(SimpleGladeApp):
         if width_percents == 100 and height_percents == 100:
             self.printDebug("MAXIMIZING MAIN WINDOW")
             self.window.maximize()
-        else:
+        elif not self.is_fullscreen:
             self.printDebug("RESIZING MAIN WINDOW TO THE FOLLOWING VALUES:")
             self.window.unmaximize()
             self.printDebug("  window_rect.x: %s", window_rect.x)
@@ -1123,6 +1147,7 @@ class Guake(SimpleGladeApp):
         self.client.notify(KEY('/style/background/image'))
         self.client.notify(KEY('/style/background/transparency'))
         self.client.notify(KEY('/general/use_default_font'))
+        self.client.notify(KEY('/general/show_resizer'))
         self.client.notify(KEY('/general/use_palette_font_and_background_color'))
         self.client.notify(KEY('/general/compat_backspace'))
         self.client.notify(KEY('/general/compat_delete'))
@@ -1333,6 +1358,7 @@ class Guake(SimpleGladeApp):
         return True
 
     def fullscreen(self):
+        self.printDebug("FULLSCREEN: ON")
         self.window.fullscreen()
         self.is_fullscreen = True
 
@@ -1346,9 +1372,11 @@ class Guake(SimpleGladeApp):
     def unfullscreen(self):
 
         # Fixes "Guake cannot restore from fullscreen" (#628)
+        self.printDebug("UNMAXIMIZING")
         self.window.unmaximize()
 
         self.set_final_window_rect()
+        self.printDebug("FULLSCREEN: OFF")
         self.window.unfullscreen()
         self.is_fullscreen = False
 
@@ -1392,7 +1420,7 @@ class Guake(SimpleGladeApp):
             current_directory = vte.get_current_directory()
             if self.abbreviate and vte_title.endswith(current_directory):
                 parts = current_directory.split('/')
-                parts = list(map(lambda s: s[:1], parts[:-1])) + [parts[-1]]
+                parts = [s[:1] for s in parts[:-1]] + [parts[-1]]
                 vte_title = vte_title[:len(vte_title) - len(current_directory)] + '/'.join(parts)
         except OSError:
             pass
@@ -1415,7 +1443,9 @@ class Guake(SimpleGladeApp):
         tab = self.tabs.get_children()[page]
         # if tab has been renamed by user, don't override.
         if not getattr(tab, 'custom_label_set', False):
-            tab.set_label(self.compute_tab_title(vte))
+            vte_title = self.compute_tab_title(vte)
+            tab.set_label(vte_title)
+            gtk.Tooltips().set_tip(tab, vte_title)
 
     def on_rename_current_tab_activate(self, *args):
         """Shows a dialog to rename the current tab.
@@ -1535,6 +1565,8 @@ class Guake(SimpleGladeApp):
         else:
             tab.set_label(new_text)
             setattr(tab, 'custom_label_set', new_text != "-")
+            if new_text != "-":
+                setattr(self.selected_tab, 'custom_label_text', new_text)
             terminals = self.notebook.get_terminals_for_tab(tab_index)
             for current_vte in terminals:
                 current_vte.emit('window-title-changed')
@@ -1549,6 +1581,8 @@ class Guake(SimpleGladeApp):
         else:
             tab.set_label(new_text)
             setattr(tab, 'custom_label_set', new_text != "-")
+            if new_text != "-":
+                setattr(self.selected_tab, 'custom_label_text', new_text)
             terminals = self.notebook.get_terminals_for_tab(tab_index)
             for current_vte in terminals:
                 current_vte.emit('window-title-changed')
@@ -1557,6 +1591,7 @@ class Guake(SimpleGladeApp):
         """Sets the `self.selected_tab' var with the selected radio
         button and change its label to `new_text'.
         """
+
         pagepos = self.notebook.get_current_page()
         self.selected_tab = self.tabs.get_children()[pagepos]
         self.selected_tab.set_label(new_text)
@@ -1564,6 +1599,8 @@ class Guake(SimpleGladeApp):
         # it's hard to pass an empty string as a command line argument,
         # so we'll interpret single dash "-" as a "reset custom title" request
         setattr(self.selected_tab, 'custom_label_set', new_text != "-")
+        if new_text != "-":
+            setattr(self.selected_tab, 'custom_label_text', new_text)
 
         # trigger titling handler in case that custom label has been reset
         current_vte = self.notebook.get_current_terminal()
@@ -1607,13 +1644,13 @@ class Guake(SimpleGladeApp):
             params['directory'] = self.get_current_dir()
         params['loglastlog'] = login_shell
 
-        # Leting caller change/add values to fork params.
+        # Letting caller change/add values to fork params.
         if default_params:
             params.update(default_params)
 
         # Environment variables are not actually parameters but they
         # need to be set before calling terminal.fork_command()
-        # method. So I found this place good to do it.
+        # method. This is a good place to do it.
         self.update_proxy_vars(box)
         return params
 
@@ -1638,13 +1675,13 @@ class Guake(SimpleGladeApp):
                 auth_pass = self.client.get_string(
                     proxy + 'authentication_password')
                 auth_pass = quote_plus(auth_pass, '')
-                os.environ['http_proxy'] = 'http://%s:%s@%s:%d' % (
+                os.environ['http_proxy'] = "http://{!s}:{!s}@{!s}:{:d}".format(
                     auth_user, auth_pass, host, port)
-                os.environ['https_proxy'] = 'http://%s:%s@%s:%d' % (
+                os.environ['https_proxy'] = "http://{!s}:{!s}@{!s}:{:d}".format(
                     auth_user, auth_pass, ssl_host, ssl_port)
             else:
-                os.environ['http_proxy'] = 'http://%s:%d' % (host, port)
-                os.environ['https_proxy'] = 'http://%s:%d' % (
+                os.environ['http_proxy'] = "http://{!s}:{:d}".format(host, port)
+                os.environ['https_proxy'] = "http://{!s}:{:d}".format(
                     ssl_host, ssl_port)
         if box:
             os.environ['GUAKE_TAB_UUID'] = str(box.terminal.get_uuid())
@@ -1703,10 +1740,12 @@ class Guake(SimpleGladeApp):
         bnt.set_property('can-focus', False)
         bnt.set_property('draw-indicator', False)
         bnt.connect('button-press-event', self.show_tab_menu)
-        bnt.connect('button-press-event', self.show_rename_current_tab_dialog)
-        bnt.connect('clicked', lambda *x: self.notebook.set_current_page(
+        bnt.activate_tab = lambda *x: self.notebook.set_current_page(
             self.notebook.page_num(box)
-        ))
+        )
+        bnt.connect('button-press-event', self.middle_button_click)
+        bnt.connect('button-press-event', self.show_rename_current_tab_dialog)
+        bnt.connect('clicked', bnt.activate_tab)
         if self.selected_color is not None:
             bnt.modify_bg(gtk.STATE_ACTIVE, gtk.gdk.Color(
                 str(self.selected_color)))
@@ -1720,10 +1759,14 @@ class Guake(SimpleGladeApp):
         self.tabs.pack_start(bnt, expand=False, padding=1)
 
         self.notebook.append_page(box, None)
-        self.notebook.set_current_page(self.notebook.page_num(box))
+        bnt.activate_tab()
         box.terminal.grab_focus()
         self.load_config()
 
+        for tab in self.tabs:
+            if getattr(tab, 'custom_label_set', False):
+                tab.set_label(getattr(tab, 'custom_label_text', tab.get_label()))
+
         if self.is_fullscreen:
             self.fullscreen()
 
@@ -1858,12 +1901,16 @@ class Guake(SimpleGladeApp):
             self.abbreviate = False
             self.recompute_tabs_titles()
 
+        for tab in self.tabs:
+            if getattr(tab, 'custom_label_set', False):
+                tab.set_label(getattr(tab, 'custom_label_text', tab.get_label()))
+
     def set_terminal_focus(self):
         """Grabs the focus on the current tab.
         """
         self.notebook.get_current_terminal().grab_focus()
         self.notebook.set_current_page(self.get_selected_tab())
-        # Hack to fix "Not focused on openning if tab was moved" (#441)
+        # Hack to fix "Not focused on opening if tab was moved" (#441)
         pos = self.get_selected_tab()
         self.select_tab(0)
         self.select_tab(pos)
@@ -1877,7 +1924,7 @@ class Guake(SimpleGladeApp):
 
     def select_current_tab(self, notebook, user_data, page):
         """When current self.notebook page is changed, the tab bar
-        made with radio buttons must be updated and this method does
+        made with radio buttons must be updated.  This method does
         this work.
         """
         self.tabs.get_children()[page].set_active(True)
@@ -1892,15 +1939,23 @@ class Guake(SimpleGladeApp):
             pass
 
     def get_selected_tab(self):
-        """return the selected tab index, it also set the
+        """Return the selected tab index and set the
         self.selected_tab var.
         """
         pagepos = self.notebook.get_current_page()
         self.selected_tab = self.tabs.get_children()[pagepos]
         return pagepos
 
+    def get_selected_tablabel(self):
+        """Return the selected tab label.
+        """
+        pagepos = self.notebook.get_current_page()
+        tab = self.tabs.get_children()[pagepos]
+        tablabel = tab.get_label()
+        return tablabel
+
     def search_on_web(self, *args):
-        """search on web the selected text
+        """Search for the selected text on the web
         """
         current_term = self.notebook.get_current_terminal()
 
@@ -1910,7 +1965,7 @@ class Guake(SimpleGladeApp):
             search_query = guake_clipboard.wait_for_text()
             search_query = quote_plus(search_query)
             if search_query:
-                search_url = "https://www.google.com/#q=%s&safe=off" % (search_query,)
+                search_url = "https://www.google.com/#q={!s}&safe=off".format(search_query,)
                 gtk.show_uri(current_term.window.get_screen(), search_url,
                              gtk.gdk.x11_get_server_time(current_term.window))
         return True
@@ -1945,3 +2000,23 @@ class Guake(SimpleGladeApp):
         current_term = self.notebook.get_current_terminal()
         current_term.reset(full=True, clear_history=True)
         self.preventHide = False
+
+    def execute_hook(self, event_name):
+        """Execute shell commands related to current event_name"""
+        hook = self.client.get_string(KEY('/hooks/{!s}'.format(event_name)))
+        if hook is not None:
+            hook = hook.split()
+            try:
+                subprocess.Popen(hook)
+            except OSError as oserr:
+                if oserr.errno == 8:
+                    log.error("Hook execution failed! Check shebang at first line of %s!", hook)
+                    log.debug(traceback.format_exc())
+                else:
+                    log.error(str(oserr))
+            except Exception as e:
+                log.error("hook execution failed! %s", e)
+                log.debug(traceback.format_exc())
+            else:
+                log.debug("hook on event %s has been executed", event_name)
+        return
